/*
To do:

T = needs testing
X = done
- = cancelled

[T] register Ham_Killed instead of using zpnm killed native
[T] CZ bots support
[T] fix possible overflow
[T] remove unnecessary checks
[T] remove zpnm alive/connected native checks and use default ones instead
[T] charsmax instead of constant values
[T] support for the new zpnm modes
[T] display mod as unknown or not at all if it's out of bounds
v1.1.9
[T] remove hud sync object from the round status hud
[T] use static hud channels instead of auto
[-] reduce hud character length from 512(hud messages seem to have a 480 character limit)
[T] display the first 10 players only to avoid filling the screen with hud
v1.2.2
[T] fix plugin not displaying players if their IDs are > 10
v1.2.3
[T] lower the hud location a bit so that dead players can properly see it too
[T] add health commas for easier health reading
v1.2.5
[X] different colors for each mode
[T] cvar when to show hud
[T] global mode id variable instead of task string info
[T] task id
[T] hud refresh interval cvar
[T] cvar hud character limit
[T] constantly display number of classes or humans/zombies left?
[T] make sure the hud doesn't exceed the max length with the extra added commas
v1.3.3
[ ] optimize
[ ] fix player names being the ones from client info instead of the ones on the server
[ ] fix hud not showing on new round
[ ] unknown mode in show hud task instead of defined modes to avoid errors
[ ] 
*/

/*================================================================================
	
		*********************************************************
		*********** [Zombie Plague Multi HUD Info] **************
		*********************************************************
	
	----------------------
	-*- Licensing Info -*-
	----------------------
	
	Zombie Plague Multi HUD Info
	by schmurgel1983(@msn.com)
	Copyright (C) 2008-2011 Stefan "schmurgel1983" Focke
	Versions 1.1.0 and higher by george.distinct(@gmail.com)
	Copyright (C) 2011-2013 D i 5 7 i n c T
	
	This program is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation, either version 3 of the License, or (at your
	option) any later version.
	
	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
	Public License for more details.
	
	You should have received a copy of the GNU General Public License along
	with this program. If not, see <http://www.gnu.org/licenses/>.
	
	In addition, as a special exception, the author gives permission to
	link the code of this program with the Half-Life Game Engine ("HL
	Engine") and Modified Game Libraries ("MODs") developed by Valve,
	L.L.C ("Valve"). You must obey the GNU General Public License in all
	respects for all of the code used other than the HL Engine and MODs
	from Valve. If you modify this file, you may extend this exception
	to your version of the file, but you are not obligated to do so. If
	you do not wish to do so, delete this exception statement from your
	version.
	
	No warranties of any kind. Use at your own risk.
	
	-------------------
	-*- Description -*-
	-------------------
	
	Shows stats like csstats with zombie/human remaining.
	Gives every 3 seconds info about the HP from first zombie,
	last human, nemesis, survivor, sniper or all of these classes.
	Shows currently played game mode.
	
	--------------------
	-*- Requirements -*-
	--------------------
	
	* Mods: Counter-Strike 1.6 or Condition-Zero
	* AMXX: Version 1.8.0 or later
	* Module: fakemeta, hamsandwich
	
	-----------------
	-*- Changelog -*-
	-----------------
	
* v1.0.0:
   - Initial release Privat (30th Aug 2008)
   - Initial release Alliedmodders (2nd Feb 2011)
	
* v1.1.0 (1st Nov 2011 - Private):
   - Fully rescripted the plug-in (optimized very much)
   - Removed the hamsandwich module and support for CZ bots
   - Added support for Zombie Plague Nightmare (version 1.9f or later)
	
* v1.1.1 (17th Nov 2011 - Private):
   - Renamed plug-in from "Zombie Plague Stats" to "Zombie Plague Multi HUD Info"
		(Thanks to suNx for the inspiration and idea)
   - Fixed human and zombie count error
   - Added support for showing the current game mode and different colors
   - Added multilingual support
   - Optimized the plug-in a bit more and removed useless code
	
=================================================================================*/

// Modules
#include <amxmodx>
#include <hamsandwich>

#if AMXX_VERSION_NUM < 180
	#assert AMX Mod X v1.8.0 or later library required!
#endif

// Plug-in APIs
#include <cs_ham_bots_api>
#include <zombieplaguenightmare>

/*================================================================================
 [Global Variables]
=================================================================================*/

// Game vars
new g_iMaxPlayers // max player counter
//new g_HudSync // message sync objects
new g_zombie_count // zombie count
new g_human_count // human count
new g_name[33][32] // player name

new const g_szLangKeysModes[][] = 
{
	"ZPNM_MODE_NONE",
	"ZPNM_MODE_INFECTION",
	"ZPNM_MODE_NEMESIS",
	"ZPNM_MODE_SURVIVOR",
	"ZPNM_MODE_SWARM",
	"ZPNM_MODE_MULTI",
	"ZPNM_MODE_PLAGUE",
	"ZPNM_MODE_SNIPER",
	"ZPNM_MODE_ASSASSIN",
	"ZPNM_SURVIVOR_NEMESIS",
	"ZPNM_MODE_SURVIVOR_ASSASSIN",
	"ZPNM_MODE_SNIPER_NEMESIS",
	"ZPNM_MODE_SNIPER_ASSASSIN",
	"ZPNM_MODE_SURVSNIP_NEMASS",
	"ZPNM_MODE_UNKNOWN"
}
const g_iSizeLangKeysModes = sizeof g_szLangKeysModes - 1;

new const g_iHUDColors[][3] =
{
	{ 0, 0, 0 },
	{ 0, 255, 0 },
	{ 255, 0, 0 },
	{ 0, 0, 255 },
	{ 255, 255, 0 },
	{ 0, 255, 0 },
	{ 255, 0, 255 },
	{ 0, 255, 255 },
	{ 175, 0, 255 },
	{ 255, 0, 255 },
	{ 175, 0, 255 },
	{ 255, 255, 255 },
	{ 175, 255, 255 },
	{ 255, 255, 255 },
	{ 255, 255, 255 }
}

/*================================================================================
 [Init]
=================================================================================*/

new g_iIDPCVarHUDShow, g_iIDPCVarHUDRefresh, g_iIDPCVarHUDLimit;

const g_iIDTaskShowHUD = 213129889;

new g_iIDMode;

enum (<<= 1)
{
	ShowOnNewRound = 1,
	ShowOnModeStart,
	ShowOnRoundEnd,
	ShowCurrentMode,
	ShowCountPlayers,
	ShowCountDead,
	ShowCountHuman,
	ShowCountZombie,
	ShowHPZombieFirst,
	ShowHPZombieLast,
	ShowHPHumanLast,
	ShowHPNemesis,
	ShowHPAssassin,
	ShowHPSurvivor,
	ShowHPSniper
}

public plugin_init()
{
	register_plugin("[ZPNM] Multi HUD Info", "1.3.3", "schmurgel1983, D i 5 7 i n c T")
	
	register_dictionary("statsx.txt")
	register_dictionary("zombie_plague.txt")
	register_dictionary("zpnm_multi_hud_info.txt")
	
	register_event("HLTV", "event_round_start", "a", "1=0", "2=0")
	
	RegisterHam(Ham_Killed, "player", "fwHamKilledPlayer", 1)
	RegisterHamBots(Ham_Killed, "fwHamKilledPlayer", 1)
	
	g_iMaxPlayers = get_maxplayers()
	
	//g_HudSync = CreateHudSyncObj()
	
	g_iIDPCVarHUDShow = register_cvar("ZPNMMultiHUDInfoShow", "bcdefghijklmno");
	g_iIDPCVarHUDRefresh = register_cvar("ZPNMMultiHUDInfoRefresh", "2.0");
	g_iIDPCVarHUDLimit = register_cvar("ZPNMMultiHUDInfoLimit", "350");
}

public client_disconnect(iID)
{
	if (!is_user_alive(iID) || zp_has_round_started() != -1)
		return;
	
	if (zp_get_user_zombie(iID))
	{
		g_zombie_count--
		
		//if (g_zombie_count)
			//display_enemy_remaining()
	}
	else
	{
		g_human_count--
		
		//if (g_human_count)
			//display_enemy_remaining()
	}
}

public client_infochanged(iID)
{
	if (!is_user_connected(iID))
		return;
	
	get_user_info(iID, "name", g_name[iID], 31)
}

/*================================================================================
 [Main Forwards]
=================================================================================*/

// Event Round Start
public event_round_start()
{
	g_iIDMode = MODE_NONE;
	
	new szCVarHUDShow[16], iBsCVarHUDShow;
	get_pcvar_string(g_iIDPCVarHUDShow, szCVarHUDShow, charsmax(szCVarHUDShow))
	iBsCVarHUDShow = read_flags(szCVarHUDShow);
	
	if (!(iBsCVarHUDShow & ShowOnNewRound))
		remove_task(g_iIDTaskShowHUD)
	else if (!task_exists(g_iIDTaskShowHUD))
	{
		fwTaskFormatShowHUD()
		set_task(get_pcvar_float(g_iIDPCVarHUDRefresh), "fwTaskFormatShowHUD", g_iIDTaskShowHUD, _, _, "b")
	}
	
	g_zombie_count = 0
	g_human_count = 0
}

public zp_round_started(mode)
{
	g_iIDMode = mode;
	
	new szCVarHUDShow[16], iBsCVarHUDShow;
	get_pcvar_string(g_iIDPCVarHUDShow, szCVarHUDShow, charsmax(szCVarHUDShow))
	iBsCVarHUDShow = read_flags(szCVarHUDShow);
	
	if (!(iBsCVarHUDShow & ShowOnModeStart))
		remove_task(g_iIDTaskShowHUD)
	else if (!task_exists(g_iIDTaskShowHUD))
	{
		fwTaskFormatShowHUD()
		set_task(get_pcvar_float(g_iIDPCVarHUDRefresh), "fwTaskFormatShowHUD", g_iIDTaskShowHUD, _, _, "b")
	}
}

public zp_round_ended()
{
	new szCVarHUDShow[16], iBsCVarHUDShow;
	get_pcvar_string(g_iIDPCVarHUDShow, szCVarHUDShow, charsmax(szCVarHUDShow))
	iBsCVarHUDShow = read_flags(szCVarHUDShow);
	
	if (!(iBsCVarHUDShow & ShowOnRoundEnd))
		remove_task(g_iIDTaskShowHUD)
	else if (!task_exists(g_iIDTaskShowHUD))
	{
		fwTaskFormatShowHUD()
		set_task(get_pcvar_float(g_iIDPCVarHUDRefresh), "fwTaskFormatShowHUD", g_iIDTaskShowHUD, _, _, "b")
	}
}

public zpnm_user_spawn_post(iID)
{
	g_human_count++
	
	//if (zp_has_round_started() == 1)
		//display_enemy_remaining()
}

public zp_user_infected_pre(iID)
{
	if (zp_get_user_zombie(iID))
		return;
	
	g_human_count--
	g_zombie_count++
	
	//display_enemy_remaining()
}

public zp_user_humanized_pre(iID)
{
	if (!zp_get_user_zombie(iID))
		return;
	
	g_human_count++
	g_zombie_count--
	
	//display_enemy_remaining()
}

public fwHamKilledPlayer(const iID)
{
	if (zp_get_user_zombie(iID))
	{
		g_zombie_count--
		
		//if (g_zombie_count)
			//display_enemy_remaining()
	}
	else
	{
		g_human_count--
		
		//if (zp_has_round_started() == 1 && g_human_count)
			//display_enemy_remaining()
	}
}

/*================================================================================
 [Other Functions]
=================================================================================*/

ftNumberAddCommasI(iNumber, szNumber[14] = "", const iSize = charsmax(szNumber))
{
	num_to_str(iNumber, szNumber, iSize);
	
	if (-1000 < iNumber < 1000)
		return szNumber;
	
	for (iNumber = strlen(szNumber) - 3; iNumber > ((szNumber[0] != '-') ? 0 : 1); iNumber -= 3)
		format(szNumber[iNumber], charsmax(szNumber) - iNumber + 1, ",%s", szNumber[iNumber]);
	
	return szNumber;
}

public fwTaskFormatShowHUD()
{
	//480 max hud length apparently
	new szMessage[512];
	
	new szCVarHUDShow[16], iBsCVarHUDShow, iCVarHUDLimit = get_pcvar_num(g_iIDPCVarHUDLimit);
	get_pcvar_string(g_iIDPCVarHUDShow, szCVarHUDShow, charsmax(szCVarHUDShow))
	iBsCVarHUDShow = read_flags(szCVarHUDShow);
	
	if (iCVarHUDLimit > 480)
		iCVarHUDLimit = 480;
	
	new iPos, iPosLast;
	
	if (iBsCVarHUDShow & ShowCurrentMode)
	{
		iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L: %L",
		LANG_SERVER, "ZPNM_CURRENT_MODE", LANG_SERVER, g_szLangKeysModes[g_iIDMode]);
		
		if (iPos > iCVarHUDLimit)
		{
			szMessage[iPosLast] = EOS;
			
			goto LabelShowHUD;
		}
		else
			iPosLast = iPos;
	}
		
	if (iBsCVarHUDShow & ShowCountPlayers)
	{
		iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%s[%L: %d]",
		(iBsCVarHUDShow & ShowCurrentMode) ? "^n" : "", LANG_SERVER, "PLAYERS", get_playersnum());
		
		if (iPos > iCVarHUDLimit)
		{
			szMessage[iPosLast] = EOS;
			
			goto LabelShowHUD;
		}
		else
			iPosLast = iPos;
	}
	
	if (iBsCVarHUDShow & ShowCountDead)
	{
		iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%s[%L: %d]",
		(iBsCVarHUDShow & ShowCurrentMode && !(iBsCVarHUDShow & ShowCountPlayers)) ? "^n" : "",
		LANG_SERVER, "ZPNM_DEAD_ADJECTIVE_PLURAL", get_playersnum() - g_human_count - g_zombie_count);
		
		if (iPos > iCVarHUDLimit)
		{
			szMessage[iPosLast] = EOS;
			
			goto LabelShowHUD;
		}
		else
			iPosLast = iPos;
	}
	
	if (iBsCVarHUDShow & ShowCountHuman)
	{
		iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%s[%L: %d]",
		(iBsCVarHUDShow & ShowCurrentMode && !(iBsCVarHUDShow & (ShowCountPlayers | ShowCountDead))) ? "^n" : "",
		LANG_SERVER, "MENU_INFO2", g_human_count);
		
		if (iPos > iCVarHUDLimit)
		{
			szMessage[iPosLast] = EOS;
			
			goto LabelShowHUD;
		}
		else
			iPosLast = iPos;
	}
	
	if (iBsCVarHUDShow & ShowCountZombie)
	{
		iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%s[%L: %d]",
		(iBsCVarHUDShow & ShowCurrentMode && !(iBsCVarHUDShow & (ShowCountPlayers | ShowCountDead | ShowCountHuman))) ? "^n" : "",
		LANG_SERVER, "MENU_INFO3", g_zombie_count);
		
		if (iPos > iCVarHUDLimit)
		{
			szMessage[iPosLast] = EOS;
			
			goto LabelShowHUD;
		}
		else
			iPosLast = iPos;
	}
	
	iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "^n^n")
	
	if (iPos > iCVarHUDLimit)
	{
		szMessage[iPosLast] = EOS;
		
		goto LabelShowHUD;
	}
	else
		iPosLast = iPos;
	
	for (new i = 1; i <= g_iMaxPlayers; i++)
	{
		if (!is_user_alive(i))
			continue;
		
		if (iBsCVarHUDShow & ShowHPZombieFirst && zp_get_user_first_zombie(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_FIRST_ZOMBIE", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else if (iBsCVarHUDShow & ShowHPHumanLast && zp_get_user_last_human(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_LAST_HUMAN", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else if (iBsCVarHUDShow & ShowHPZombieLast && zp_get_user_last_zombie(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_LAST_ZOMBIE", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else if (iBsCVarHUDShow & ShowHPNemesis && zp_get_user_nemesis(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_NEMESIS", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else if (iBsCVarHUDShow & ShowHPSurvivor && zp_get_user_survivor(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_SURVIVOR", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else if (iBsCVarHUDShow & ShowHPSniper && zpnm_get_user_sniper(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_SNIPER", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else if (iBsCVarHUDShow & ShowHPAssassin && zpnm_get_user_assassin(i))
		{
			iPos += formatex(szMessage[iPos], charsmax(szMessage) - iPos, "%L^n", LANG_SERVER, "ZPNM_ASSASSIN", g_name[i], ftNumberAddCommasI(get_user_health(i)))
			
			if (iPos > iCVarHUDLimit)
			{
				szMessage[iPosLast] = EOS;
				
				break;
			}
			else
				iPosLast = iPos;
		}
		else
			continue;
		
/*		
		if (is_user_alive(i))
//			set_hudmessage(g_hud_color_R[g_iIDMode], g_hud_color_G[g_iIDMode], g_hud_color_B[g_iIDMode], -1.0, 0.0, 0, 0.01, 3.57, 0.01, 0.01, -1)
			set_hudmessage(0, 255, 255, -1.0, 0.0, 0, 0.01, 3.57, 0.01, 0.01, -1)
		else
//			set_hudmessage(g_hud_color_R[g_iIDMode], g_hud_color_G[g_iIDMode], g_hud_color_B[g_iIDMode], -1.0, 0.357, 0, 0.01, 3.57, 0.01, 0.01, -1)
			set_hudmessage(0, 255, 255, -1.0, 0.357, 0, 0.01, 3.57, 0.01, 0.01, -1)
		
		ShowSyncHudMsg(i, g_HudSync, "%s", szMessage)*/
	}
	
	LabelShowHUD:
	
	new Float:fCVarRefresh = get_pcvar_float(g_iIDPCVarHUDRefresh);
	
	set_hudmessage(g_iHUDColors[g_iIDMode][0], g_iHUDColors[g_iIDMode][1], g_iHUDColors[g_iIDMode][2], -1.0, 0.14, 0, 0.0, fCVarRefresh + ((fCVarRefresh >= 0.2) ? 0.1 : 0.05), 0.0, 0.0, 3)
	show_hudmessage(0, szMessage)
}
/*
public display_enemy_remaining()
{
	static message[128]
	
	set_hudmessage(175, 175, 175, 0.0157, 0.975, 2, 0.02, 3.57, 0.01, 1.57, 4)
	
	for (new i = 1; i <= g_iMaxPlayers; i++)
	{
		if (!is_user_connected(i))
			continue;
		
		if (!zp_get_user_zombie(i))
		{
			formatex(message, charsmax(message), "%L", i, "ZPNM_ZOMBIES_REMAINING", g_zombie_count)
			ShowSyncHudMsg(i, g_HudSync, "%s", message)
		}
		else
		{
			formatex(message, charsmax(message), "%L", i, "ZPNM_HUMANS_REMAINING", g_human_count)
			ShowSyncHudMsg(i, g_HudSync, "%s", message)
		}
	}
}
*/